@@ -59,7 +59,9 @@ eXosip_masquerade_contact (const char *public_address, int port)
   eXtl_tcp.tl_masquerade_contact(public_address, port);
 #ifdef HAVE_OPENSSL_SSL_H
   eXtl_tls.tl_masquerade_contact(public_address, port);
+#if !(OPENSSL_VERSION_NUMBER < 0x00908000L)
   eXtl_dtls.tl_masquerade_contact(public_address, port);
+#endif
 #endif
   return;
 }
@@ -256,10 +258,27 @@ eXosip_quit (void)
       osip_free (jauthinfo);
     }
 
+  {
+	struct eXosip_http_auth *http_auth;
+	int pos;
+
+	/* update entries with same call_id */
+	for (pos=0;pos<MAX_EXOSIP_HTTP_AUTH;pos++)
+	{
+		http_auth = &eXosip.http_auths[pos];
+		if (http_auth->pszCallId[0]=='\0')
+			continue;
+		osip_proxy_authenticate_free(http_auth->wa);
+		memset(http_auth, 0, sizeof(struct eXosip_http_auth));
+	}
+  }
+
   eXtl_udp.tl_free();
   eXtl_tcp.tl_free();
 #ifdef HAVE_OPENSSL_SSL_H
+#if !(OPENSSL_VERSION_NUMBER < 0x00908000L)
   eXtl_dtls.tl_free();
+#endif
   eXtl_tls.tl_free();
 #endif
 
@@ -564,8 +583,10 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
   else if (transport==IPPROTO_TCP && secure==0)
     eXtl = &eXtl_tcp;
 #ifdef HAVE_OPENSSL_SSL_H
+#if !(OPENSSL_VERSION_NUMBER < 0x00908000L)
   else if (transport==IPPROTO_UDP)
     eXtl = &eXtl_dtls;
+#endif
   else if (transport==IPPROTO_TCP)
     eXtl = &eXtl_tls;
 #endif
@@ -698,7 +719,9 @@ eXosip_init (void)
   eXtl_udp.tl_init();
   eXtl_tcp.tl_init();
 #ifdef HAVE_OPENSSL_SSL_H
+#if !(OPENSSL_VERSION_NUMBER < 0x00908000L)
   eXtl_dtls.tl_init();
+#endif
   eXtl_tls.tl_init();
 #endif
   return 0;
@@ -760,6 +783,7 @@ eXosip_execute (void)
   /* free all Calls that are in the TERMINATED STATE? */
   eXosip_release_terminated_calls ();
   eXosip_release_terminated_registrations ();
+  eXosip_release_terminated_publications ();
 #ifndef MINISIZE
   eXosip_release_unused_transactions();
 #endif
