@@ -80,7 +80,7 @@
 
 #endif
 
-#define EXOSIP_VERSION	"3.0.3"
+#define EXOSIP_VERSION	"3.1.0"
 
 #ifdef __cplusplus
 extern "C"
@@ -156,6 +156,7 @@ void eXosip_freeaddrinfo(struct addrinfo *ai);
     osip_list_t *d_inc_trs;
     osip_list_t *d_out_trs;
     int d_retry;                /* avoid too many unsuccessfull retry */
+    int d_mincseq; /* remember cseq after PRACK and UPDATE during setup */
 
     eXosip_dialog_t *next;
     eXosip_dialog_t *parent;
@@ -216,9 +217,6 @@ void eXosip_freeaddrinfo(struct addrinfo *ai);
     int c_retry;                /* avoid too many unsuccessfull retry */
     void *external_reference;
 
-    /* cache old credentials for reuse in next request */
-    osip_message_t *response_auth;
-
     eXosip_call_t *next;
     eXosip_call_t *parent;
   };
@@ -257,12 +255,12 @@ void eXosip_freeaddrinfo(struct addrinfo *ai);
   {
     int p_id;
 
-    time_t p_expires;           /* expiration date (started+period) */
     int p_period;               /* delay between registration */
     char p_aor[256];            /* sip identity */
     char p_sip_etag[64];        /* sip_etag from 200ok */
 
     osip_transaction_t *p_last_tr;
+    int p_retry;
     eXosip_pub_t *next;
     eXosip_pub_t *parent;
   };
@@ -308,7 +306,7 @@ void eXosip_freeaddrinfo(struct addrinfo *ai);
                                                   ** auth, const char *method,
 												  const char *pszCNonce,
 												  int iNonceCount);
-  int _eXosip_store_nonce(const char *call_id, osip_proxy_authenticate_t *wa);
+  int _eXosip_store_nonce(const char *call_id, osip_proxy_authenticate_t *wa, int answer_code);
   int _eXosip_delete_nonce(const char *call_id);
 
   eXosip_event_t *eXosip_event_init_for_call (int type, eXosip_call_t * jc,
@@ -404,7 +402,7 @@ void eXosip_freeaddrinfo(struct addrinfo *ai);
 #endif
 
 #ifndef MAX_EXOSIP_HTTP_AUTH
-#define MAX_EXOSIP_HTTP_AUTH 10
+#define MAX_EXOSIP_HTTP_AUTH 100
 #endif
 
   typedef struct eXosip_t eXosip_t;
@@ -632,6 +630,7 @@ void eXosip_freeaddrinfo(struct addrinfo *ai);
   void eXosip_release_terminated_calls (void);
   void eXosip_release_terminated_registrations (void);
   void eXosip_release_unused_transactions(void);
+  void eXosip_release_terminated_publications (void);
 
 #ifndef MINISIZE
   int eXosip_subscribe_init (eXosip_subscribe_t ** js);
