@@ -28,6 +28,9 @@
 #ifdef HAVE_OPENSSL_SSL_H
 
 #include <openssl/ssl.h>
+
+#if !(OPENSSL_VERSION_NUMBER < 0x00908000L)
+
 #define SPROTO_TLS 500
 #define SPROTO_DTLS 501
 #include <openssl/ssl.h>
@@ -1008,7 +1011,7 @@ dtls_tl_send_message(osip_transaction_t * tr, osip_message_t * sip, char *host,
 
 #ifndef MINISIZE
       /* delete first SRV entry that is not reachable */
-      if (tr->record.name[0]!='\0' && tr->record.srventry[0].srv[0]!='\0')
+      if (tr!=NULL && tr->record.name[0]!='\0' && tr->record.srventry[0].srv[0]!='\0')
 	{
 	  memmove(&tr->record.srventry[0], &tr->record.srventry[1], 9*sizeof(osip_srv_entry_t));
 	  memset(&tr->record.srventry[9], 0, sizeof(osip_srv_entry_t));
@@ -1122,3 +1125,5 @@ struct eXtl_protocol eXtl_dtls =
 };
 
 #endif
+
+#endif
