@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002, 2003  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -91,8 +91,14 @@ eXosip_call_free (eXosip_call_t * jc)
 
   eXosip_dialog_t *jd;
 
-  if (jc->response_auth != NULL)
-    osip_message_free (jc->response_auth);
+  if (jc->c_inc_tr!=NULL && jc->c_inc_tr->orig_request!=NULL
+	  && jc->c_inc_tr->orig_request->call_id!=NULL
+	  && jc->c_inc_tr->orig_request->call_id->number!=NULL)
+	  _eXosip_delete_nonce(jc->c_inc_tr->orig_request->call_id->number);
+  else if (jc->c_out_tr!=NULL && jc->c_out_tr->orig_request!=NULL
+	  && jc->c_out_tr->orig_request->call_id!=NULL
+	  && jc->c_out_tr->orig_request->call_id->number!=NULL)
+	  _eXosip_delete_nonce(jc->c_out_tr->orig_request->call_id->number);
 
   for (jd = jc->c_dialogs; jd != NULL; jd = jc->c_dialogs)
     {
@@ -107,13 +113,6 @@ eXosip_call_free (eXosip_call_t * jc)
   if (jc->c_out_tr != NULL)
     osip_list_add (eXosip.j_transactions, jc->c_out_tr, 0);
 
-  __eXosip_delete_jinfo (jc->c_inc_options_tr);
-  __eXosip_delete_jinfo (jc->c_out_options_tr);
-  if (jc->c_inc_options_tr != NULL)
-    osip_list_add (eXosip.j_transactions, jc->c_inc_options_tr, 0);
-  if (jc->c_out_options_tr != NULL)
-    osip_list_add (eXosip.j_transactions, jc->c_out_options_tr, 0);
-
   osip_free (jc);
 
 }
