@@ -51,6 +51,10 @@
 #include <eXosip2/eXosip.h>
 #include "eXosip2.h"
 
+#ifdef HAVE_OPENSSL_SSL_H
+#include <openssl/ssl.h>
+#endif
+
 extern eXosip_t eXosip;
 
 
@@ -209,7 +213,10 @@ cb_snd_message (osip_transaction_t * tr, osip_message_t * sip, char *host,
     }
   else if (osip_strcasecmp (via->protocol, "dtls-udp") == 0)
     {
+      i=-1;
+#if !(OPENSSL_VERSION_NUMBER < 0x00908000L)
       i = eXtl_dtls.tl_send_message (tr, sip, host, port, out_socket);
+#endif
     }
 #endif
   if (i != 0)
@@ -1099,6 +1106,27 @@ cb_rcv1xx (int type, osip_transaction_t * tr, osip_message_t * sip)
 		  /* Update only if it is the same dialog */
 		  osip_dialog_update_route_set_as_uac (jd->d_dialog, sip);
 		}
+#if 1 /* to be tested */
+	      else
+		{
+		  /* the best thing is to replace the current dialog
+		     information... Much easier than creating a useless dialog! */
+		  osip_dialog_free (jd->d_dialog);
+		  i = osip_dialog_init_as_uac (&(jd->d_dialog), sip);
+		  if (i != 0)
+		    {
+		      OSIP_TRACE (osip_trace
+				  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+				   "Cannot replace the dialog.\r\n"));
+		    }
+		  else
+		    {
+		      OSIP_TRACE (osip_trace
+				  (__FILE__, __LINE__, OSIP_WARNING, NULL,
+				   "The dialog has been replaced with the new one from 1xx.\r\n"));
+		    }
+		}
+#endif
 	    }
         }
 
@@ -1227,11 +1255,14 @@ cb_rcv2xx_4invite (osip_transaction_t * tr, osip_message_t * sip)
               OSIP_TRACE (osip_trace
                           (__FILE__, __LINE__, OSIP_ERROR, NULL,
                            "Cannot replace the dialog.\r\n"));
-          } else
+	    }
+	  else
             {
+	      jd->d_dialog->local_cseq = jd->d_dialog->local_cseq + jd->d_mincseq;
+	      jd->d_mincseq=0;
               OSIP_TRACE (osip_trace
                           (__FILE__, __LINE__, OSIP_WARNING, NULL,
-                           "The dialog has been replaced with the new one fro 200ok.\r\n"));
+                           "The dialog has been replaced with the new one from 200ok.\r\n"));
             }
         }
     }
@@ -1409,7 +1440,7 @@ cb_rcv2xx (int type, osip_transaction_t * tr, osip_message_t * sip)
 #ifndef MINISIZE
   if (MSG_IS_RESPONSE_FOR (sip, "PUBLISH"))
     {
-      eXosip_pub_t *pub;
+      eXosip_pub_t *pub=NULL;
       eXosip_event_t *je;
       int i;
       
@@ -1421,6 +1452,30 @@ cb_rcv2xx (int type, osip_transaction_t * tr, osip_message_t * sip)
                        "cb_rcv2xx (id=%i) No publication to update\r\n",
                        tr->transactionid));
         }
+      if (pub != NULL)
+        {
+	  /* update registration interval */
+	  osip_header_t *exp;
+	  
+	  osip_message_header_get_byname (sip, "expires", 0, &exp);
+	  if (exp!=NULL && exp->hvalue!=NULL)
+	    {
+	      int val = atoi(exp->hvalue);
+	      if (val>0)
+		{
+		  /* update only if expires value has REALLY be
+		     decreased (more than one minutes):
+		     In many cases value is decreased because a few seconds has
+		     elapsed when server send the 200ok. */
+			if (val < pub->p_period-60)
+		    {
+		      pub->p_period=val+60;
+		    }
+		}
+	    }
+          pub->p_retry = 0;    /* reset value */
+	}
+
       je = eXosip_event_init_for_message (EXOSIP_MESSAGE_ANSWERED, tr);
       report_event (je, sip);
       return;
@@ -1445,7 +1500,14 @@ cb_rcv2xx (int type, osip_transaction_t * tr, osip_message_t * sip)
 	      int val = atoi(exp->hvalue);
 	      if (val>0)
 		{
-		  jreg->r_reg_period=val+60;
+		  /* update only if expires value has REALLY be
+		     decreased (more than one minutes):
+		     In many cases value is decreased because a few seconds has
+		     elapsed when server send the 200ok. */
+		  if (val < jreg->r_reg_period-60)
+		    {
+		      jreg->r_reg_period=val+60;
+		    }
 		}
 	    }
 
@@ -1498,7 +1560,7 @@ cb_rcv2xx (int type, osip_transaction_t * tr, osip_message_t * sip)
       osip_header_t *sub_state;
 
       je =
-        eXosip_event_init_for_notify (EXOSIP_NOTIFICATION_REQUESTFAILURE,
+        eXosip_event_init_for_notify (EXOSIP_NOTIFICATION_ANSWERED,
                                          jn, jd, tr);
       report_event (je, sip);
 
@@ -1636,7 +1698,7 @@ cb_rcv3xx (int type, osip_transaction_t * tr, osip_message_t * sip)
       eXosip_event_t *je;
 
       je =
-        eXosip_event_init_for_notify (EXOSIP_NOTIFICATION_REQUESTFAILURE,
+        eXosip_event_init_for_notify (EXOSIP_NOTIFICATION_REDIRECTED,
                                          jn, jd, tr);
       report_event (je, sip);
     }
@@ -1761,16 +1823,6 @@ cb_rcv4xx (int type, osip_transaction_t * tr, osip_message_t * sip)
       return;
     }
 
-  if (jc != NULL)
-    {
-      if (MSG_TEST_CODE (sip, 401) || MSG_TEST_CODE (sip, 407))
-        {
-          if (jc->response_auth != NULL)
-            osip_message_free (jc->response_auth);
-
-          osip_message_clone (sip, &jc->response_auth);
-        }
-    }
   if (jd == NULL)
     return;
   if (MSG_IS_RESPONSE_FOR (sip, "INVITE")
@@ -1841,7 +1893,7 @@ cb_rcv5xx (int type, osip_transaction_t * tr, osip_message_t * sip)
       eXosip_event_t *je;
 
       je =
-        eXosip_event_init_for_notify (EXOSIP_NOTIFICATION_REQUESTFAILURE,
+        eXosip_event_init_for_notify (EXOSIP_NOTIFICATION_SERVERFAILURE,
                                          jn, jd, tr);
       report_event (je, sip);
     }
@@ -1936,7 +1988,7 @@ cb_rcv6xx (int type, osip_transaction_t * tr, osip_message_t * sip)
       eXosip_event_t *je;
 
       je =
-        eXosip_event_init_for_notify (EXOSIP_NOTIFICATION_REQUESTFAILURE,
+        eXosip_event_init_for_notify (EXOSIP_NOTIFICATION_GLOBALFAILURE,
                                          jn, jd, tr);
       report_event (je, sip);
     }
@@ -2025,17 +2077,6 @@ cb_rcv3456xx (int type, osip_transaction_t * tr, osip_message_t * sip,
       return;
     }
 
-  if (jc != NULL)
-    {
-      if (MSG_TEST_CODE (sip, 401) || MSG_TEST_CODE (sip, 407))
-        {
-          if (jc->response_auth != NULL)
-            osip_message_free (jc->response_auth);
-
-          osip_message_clone (sip, &jc->response_auth);
-        }
-    }
-
   if (jd == NULL)
     return;
   if (MSG_IS_RESPONSE_FOR (sip, "INVITE"))
