@@ -424,6 +424,7 @@ eXosip_event_wait (int tv_s, int tv_ms)
   eXosip_lock();
   eXosip_retransmit_lost200ok();
   eXosip_unlock();
+
   return NULL;
 }
 
@@ -484,17 +485,34 @@ eXosip_event_wait (int tv_s, int tv_ms)
     FD_SET (jpipe_get_read_descr (eXosip.j_socketctl_event), &fdset);
 #endif
     max = jpipe_get_read_descr (eXosip.j_socketctl_event);
-    tv.tv_sec = tv_s;
-    tv.tv_usec = tv_ms * 1000;
 
     je = (eXosip_event_t *) osip_fifo_tryget (eXosip.j_events);
     if (je != NULL)
       return je;
 
-	eXosip_lock();
-	eXosip_retransmit_lost200ok();
-	eXosip_unlock();
+    tv.tv_sec = 0;
+    tv.tv_usec = 0;
+    i = select (max + 1, &fdset, NULL, NULL, &tv);
+    if (FD_ISSET (jpipe_get_read_descr (eXosip.j_socketctl_event), &fdset))
+      {
+	char buf[500];
+	jpipe_read (eXosip.j_socketctl_event, buf, 499);
+      }
+    
+    eXosip_lock();
+    eXosip_retransmit_lost200ok();
+    eXosip_unlock();
 
+    FD_ZERO (&fdset);
+#if defined (WIN32) || defined (_WIN32_WCE)
+    FD_SET ((unsigned int) jpipe_get_read_descr (eXosip.j_socketctl_event),
+	    &fdset);
+#else
+    FD_SET (jpipe_get_read_descr (eXosip.j_socketctl_event), &fdset);
+#endif
+    tv.tv_sec = tv_s;
+    tv.tv_usec = tv_ms * 1000;
+    
     if (tv_s == 0 && tv_ms == 0)
       return NULL;
 
@@ -521,6 +539,12 @@ eXosip_event_wait (int tv_s, int tv_ms)
   return je;
 }
 
+int
+eXosip_event_geteventsocket(void)
+{
+  return jpipe_get_read_descr (eXosip.j_socketctl_event);
+}
+
 eXosip_event_t *
 eXosip_event_get ()
 {
