@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002, 2003  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include <mpatrol.h>
 #endif
 
+#ifndef MINISIZE
 
 #include "eXosip2.h"
 
@@ -82,34 +83,9 @@ int
 eXosip_notify_init (eXosip_notify_t ** jn, osip_message_t * inc_subscribe)
 {
   osip_contact_t *co;
-  char *uri;
-  int i;
-  char locip[50];
 
-#ifdef SM
-  eXosip_get_localip_from_via (inc_subscribe, locip, 49);
-#else
-  i = _eXosip_find_protocol (inc_subscribe);
-  if (i == IPPROTO_UDP)
-    {
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[0].net_ip_family, locip, 49);
-  } else if (i == IPPROTO_TCP)
-    {
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[1].net_ip_family, locip, 49);
-  } else
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: unsupported protocol (default to UDP)\n"));
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[0].net_ip_family, locip, 49);
-      return -1;
-    }
-
-#endif
-  if (inc_subscribe == NULL
-      || inc_subscribe->to == NULL || inc_subscribe->to->url == NULL)
-    return -1;
-  co = (osip_contact_t *) osip_list_get (inc_subscribe->contacts, 0);
+  *jn=NULL;
+  co = (osip_contact_t *) osip_list_get (&inc_subscribe->contacts, 0);
   if (co == NULL || co->url == NULL)
     return -1;
 
@@ -118,16 +94,6 @@ eXosip_notify_init (eXosip_notify_t ** jn, osip_message_t * inc_subscribe)
     return -1;
   memset (*jn, 0, sizeof (eXosip_notify_t));
 
-  i = osip_uri_to_str (co->url, &uri);
-  if (i != 0)
-    {
-      osip_free (*jn);
-      *jn = NULL;
-      return -1;
-    }
-  osip_strncpy ((*jn)->n_uri, uri, 254);
-  osip_free (uri);
-
   return 0;
 }
 
@@ -138,6 +104,15 @@ eXosip_notify_free (eXosip_notify_t * jn)
 
   eXosip_dialog_t *jd;
 
+  if (jn->n_inc_tr!=NULL && jn->n_inc_tr->orig_request!=NULL
+	  && jn->n_inc_tr->orig_request->call_id!=NULL
+	  && jn->n_inc_tr->orig_request->call_id->number!=NULL)
+	  _eXosip_delete_nonce(jn->n_inc_tr->orig_request->call_id->number);
+  else if (jn->n_out_tr!=NULL && jn->n_out_tr->orig_request!=NULL
+	  && jn->n_out_tr->orig_request->call_id!=NULL
+	  && jn->n_out_tr->orig_request->call_id->number!=NULL)
+	  _eXosip_delete_nonce(jn->n_out_tr->orig_request->call_id->number);
+
   for (jd = jn->n_dialogs; jd != NULL; jd = jn->n_dialogs)
     {
       REMOVE_ELEMENT (jn->n_dialogs, jd);
@@ -198,3 +173,5 @@ _eXosip_notify_add_expires_in_2XX_for_subscribe (eXosip_notify_t * jn,
     }
   osip_message_set_expires (answer, tmp);
 }
+
+#endif
