@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002, 2003  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,9 +22,13 @@
 #include <mpatrol.h>
 #endif
 
+#ifdef OSIP_MT
+
 #include "jpipe.h"
 
-#ifndef WIN32
+#if !defined(WIN32) && !defined(__arc__)
+
+#include <fcntl.h>
 
 jpipe_t *
 jpipe ()
@@ -39,6 +43,15 @@ jpipe ()
       osip_free (my_pipe);
       return NULL;
     }
+  
+  if (fcntl(my_pipe->pipes[1], F_SETFL, O_NONBLOCK) == -1)
+    {
+      /* failed for some reason... */
+      OSIP_TRACE (osip_trace
+                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                   "cannot set O_NONBLOCK to the pipe[1]!\n"));
+    }
+  
   return my_pipe;
 }
 
@@ -112,7 +125,11 @@ jpipe ()
   my_pipe->pipes[1] = (int) socket (PF_INET, SOCK_STREAM, IPPROTO_TCP);
   if (0 > my_pipe->pipes[1])
     {
+#if defined(__arc__)
+      close (s);
+#else
       closesocket (s);
+#endif
       osip_free (my_pipe);
       return NULL;
     }
@@ -136,9 +153,15 @@ jpipe ()
     {
       OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL,
                               "Failed to bind a local socket, aborting!\n"));
+#if defined(__arc__)
+      close (s);
+      close (my_pipe->pipes[1]);
+#else
       closesocket (s);
       closesocket (my_pipe->pipes[1]);
+#endif
       osip_free (my_pipe);
+      return NULL;
     }
 
   j = listen (s, 1);
@@ -146,14 +169,33 @@ jpipe ()
     {
       OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL,
                               "Failed to listen on a local socket, aborting!\n"));
+#if defined(__arc__)
+      close (s);
+      close (my_pipe->pipes[1]);
+#else
       closesocket (s);
       closesocket (my_pipe->pipes[1]);
+#endif
       osip_free (my_pipe);
+      return NULL;
     }
 
   j = setsockopt (my_pipe->pipes[1],
                   SOL_SOCKET,
                   SO_RCVTIMEO, (const char *) &timeout, sizeof (timeout));
+#if defined(__arc__)
+  if (j != 0)
+    {
+      /* failed for some reason... */
+      OSIP_TRACE (osip_trace
+                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                   "udp plugin; cannot set O_NONBLOCK to the file desciptor!\n"));
+      close (s);
+      close (my_pipe->pipes[1]);
+      osip_free (my_pipe);
+      return NULL;
+    }
+#elif !defined(_WIN32_WCE)
   if (j != NO_ERROR)
     {
       /* failed for some reason... */
@@ -163,7 +205,9 @@ jpipe ()
       closesocket (s);
       closesocket (my_pipe->pipes[1]);
       osip_free (my_pipe);
+      return NULL;
     }
+#endif
 
   connect (my_pipe->pipes[1], (struct sockaddr *) &raddr, sizeof (raddr));
 
@@ -174,9 +218,15 @@ jpipe ()
       OSIP_TRACE (osip_trace
                   (__FILE__, __LINE__, OSIP_ERROR, NULL,
                    "udp plugin; Failed to call accept!\n"));
+#if defined(__arc__)
+      close (s);
+      close (my_pipe->pipes[1]);
+#else
       closesocket (s);
       closesocket (my_pipe->pipes[1]);
+#endif
       osip_free (my_pipe);
+      return NULL;
     }
 
   return my_pipe;
@@ -187,8 +237,13 @@ jpipe_close (jpipe_t * apipe)
 {
   if (apipe == NULL)
     return -1;
+#if defined(__arc__)
+  close (apipe->pipes[0]);
+  close (apipe->pipes[1]);
+#else
   closesocket (apipe->pipes[0]);
   closesocket (apipe->pipes[1]);
+#endif
   osip_free (apipe);
   return 0;
 }
@@ -228,3 +283,5 @@ jpipe_get_read_descr (jpipe_t * apipe)
 }
 
 #endif
+
+#endif
