@@ -203,38 +203,38 @@ eXosip_process_ack (eXosip_call_t * jc, eXosip_dialog_t * jd, osip_event_t * evt
   je = eXosip_event_init_for_call (EXOSIP_CALL_ACK, jc, jd, NULL);
   if (je != NULL)
     {
-	  osip_transaction_t *tr;
-	  tr= eXosip_find_last_inc_invite(jc, jd);
-	  if (tr!=NULL)
-	  {
-		  je->tid = tr->transactionid;
-	  }
+      osip_transaction_t *tr;
+      tr= eXosip_find_last_inc_invite(jc, jd);
+      if (tr!=NULL)
+	{
+	  je->tid = tr->transactionid;
 	  /* fill request and answer */
 	  if (tr->orig_request != NULL)
+	    {
+	      i = osip_message_clone (tr->orig_request, &je->request);
+	      if (i != 0)
 		{
-		  i = osip_message_clone (tr->orig_request, &je->request);
-		  if (i != 0)
-			{
-			  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL,
-									  "failed to clone request for event\n"));
-			}
+		  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL,
+					  "failed to clone request for event\n"));
 		}
+	    }
 	  if (tr->last_response != NULL)
+	    {
+	      i = osip_message_clone (tr->last_response, &je->response);
+	      if (i != 0)
 		{
-		  i = osip_message_clone (tr->last_response, &je->response);
-		  if (i != 0)
-			{
-			  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL,
-									  "failed to clone response for event\n"));
-			}
+		  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL,
+					  "failed to clone response for event\n"));
 		}
-
+	    }
+	}
+      
       i = osip_message_clone (evt->sip, &je->ack);
       if (i != 0)
-        {
-          OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                                  "failed to clone ACK for event\n"));
-      }
+	{
+	  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL,
+				  "failed to clone ACK for event\n"));
+	}
     }
 
   /* stop ACK retransmission, in case there is any */
@@ -774,6 +774,16 @@ eXosip_process_notify_within_dialog (eXosip_subscribe_t * js,
         je =
           eXosip_event_init_for_subscribe (EXOSIP_SUBSCRIPTION_NOTIFY, js, jd,
                                            transaction);
+		if (je->request == NULL && evt->sip != NULL)
+		{
+		  i = osip_message_clone (evt->sip, &je->request);
+		  if (i != 0)
+			{
+			  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL,
+									  "failed to clone request for event\n"));
+			}
+		}
+
         eXosip_event_add (je);
       }
 
@@ -1039,6 +1049,9 @@ eXosip_process_newrequest (osip_event_t * evt, int socket)
             }
         }
 
+      if (transaction!=NULL) /* NOT for ACK */
+	osip_dialog_update_osip_cseq_as_uas (jd->d_dialog, evt->sip);
+
       if (MSG_IS_INVITE (evt->sip))
         {
           /* the previous transaction MUST be freed */
@@ -1064,12 +1077,27 @@ eXosip_process_newrequest (osip_event_t * evt, int socket)
               return;
             }
 
-          osip_dialog_update_osip_cseq_as_uas (jd->d_dialog, evt->sip);
+          /* osip_dialog_update_osip_cseq_as_uas (jd->d_dialog, evt->sip); */
           osip_dialog_update_route_set_as_uas (jd->d_dialog, evt->sip);
 
           eXosip_process_reinvite (jc, jd, transaction, evt);
       } else if (MSG_IS_BYE (evt->sip))
         {
+	  osip_generic_param_t *tag_to=NULL;
+	  if (evt->sip->to!=NULL)
+	  osip_from_param_get_byname (evt->sip->to, "tag", &tag_to);
+	  if (tag_to == NULL || tag_to->gvalue == NULL)
+	    {
+	      OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL,
+				      "Uncompliant user agent: missing a tag in To of incoming BYE\n"));
+              osip_list_add (eXosip.j_transactions, transaction, 0);
+              eXosip_send_default_answer (jd, transaction, evt, 481,
+                                          "Missing tags in BYE",
+                                          "Missing tags in BYE",
+                                          __LINE__);
+	      return;
+	    }
+
           old_trn = eXosip_find_last_inc_transaction (jc, jd, "BYE");
 
           if (old_trn != NULL)  /* && old_trn->state!=NIST_TERMINATED) */
@@ -1090,11 +1118,6 @@ eXosip_process_newrequest (osip_event_t * evt, int socket)
       } else if (MSG_IS_PRACK (evt->sip))
         {
           eXosip_process_prack (jc, jd, transaction, evt);
-      } else if (MSG_IS_SUBSCRIBE (evt->sip))
-        {
-          osip_list_add (eXosip.j_transactions, transaction, 0);
-          eXosip_send_default_answer (jd, transaction, evt, 489, NULL,
-                                      "Bad Event", __LINE__);
       } else
         {
           eXosip_process_message_within_dialog (jc, jd, transaction, evt);
@@ -1484,7 +1507,11 @@ eXosip_process_response_out_of_transaction (osip_event_t * evt)
 	      cb_snd_message (NULL, ack, NULL,0, -1);
 	      osip_message_free(ack);
 #endif
-	      
+	      /* in some case, PRACK and UPDATE may have been sent
+		 so we have to send a cseq which is above the previous
+		 one. */
+	      dlg->local_cseq=dlg->local_cseq+4;
+
 	      /* ready to send a BYE */
 	      if (transport == NULL)
 		i = generating_bye (&bye, dlg, "UDP");
@@ -1578,7 +1605,9 @@ eXosip_read_message (int max_message_nb, int sec_max, int usec_max)
       eXtl_udp.tl_set_fdset(&osip_fdset, &max);
       eXtl_tcp.tl_set_fdset(&osip_fdset, &max);
 #ifdef HAVE_OPENSSL_SSL_H
+#if !(OPENSSL_VERSION_NUMBER < 0x00908000L)
       eXtl_dtls.tl_set_fdset(&osip_fdset, &max);
+#endif
       eXtl_tls.tl_set_fdset(&osip_fdset, &max);
 #endif
 
@@ -1624,7 +1653,9 @@ eXosip_read_message (int max_message_nb, int sec_max, int usec_max)
 	  eXtl_udp.tl_read_message(&osip_fdset);
 	  eXtl_tcp.tl_read_message(&osip_fdset);
 #ifdef HAVE_OPENSSL_SSL_H
+#if !(OPENSSL_VERSION_NUMBER < 0x00908000L)
 	  eXtl_dtls.tl_read_message(&osip_fdset);
+#endif
 	  eXtl_tls.tl_read_message(&osip_fdset);
 #endif
 	  }
@@ -1734,7 +1765,7 @@ eXosip_pendingosip_transaction_exist (eXosip_call_t * jc, eXosip_dialog_t * jd)
     }
 
   tr = eXosip_find_last_inc_transaction (jc, jd, "REFER");
-  if (tr != NULL && tr->state != IST_TERMINATED)
+  if (tr != NULL && tr->state != NIST_TERMINATED)
     {                           /* Don't want to wait forever on broken transaction!! */
       if (tr->birth_time + 180 < now)   /* Wait a max of 2 minutes */
         {
@@ -1871,7 +1902,7 @@ eXosip_release_finished_transactions (eXosip_call_t *jc, eXosip_dialog_t *jd)
 		{
 		  /* remove the transaction from oSIP */
 		  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO2, NULL,
-					  "eXosip: releaase non-INVITE server transaction (did=%i)\n",
+					  "eXosip: release non-INVITE server transaction (did=%i)\n",
 					  jd->d_id));
 		  osip_remove_transaction (eXosip.j_osip, inc_tr);
 		  osip_list_remove (jd->d_inc_trs, pos);
@@ -2282,3 +2313,37 @@ eXosip_release_terminated_registrations (void)
 
   return;
 }
+
+void
+eXosip_release_terminated_publications (void)
+{
+  eXosip_pub_t *jpub;
+  eXosip_pub_t *jpubnext;
+  time_t now = time (NULL);
+
+  for (jpub = eXosip.j_pub; jpub != NULL;)
+    {
+      jpubnext = jpub->next;
+      if (jpub->p_period == 0 && jpub->p_last_tr != NULL)
+        {
+          if (now - jpub->p_last_tr->birth_time > 60)
+            {
+              OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL,
+                                      "Release a terminated publication\n"));
+              REMOVE_ELEMENT (eXosip.j_pub, jpub);
+              _eXosip_pub_free (jpub);
+          } else if (jpub->p_last_tr->last_response != NULL
+                     && jpub->p_last_tr->last_response->status_code >= 200
+                     && jpub->p_last_tr->last_response->status_code <= 299)
+            {
+              OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL,
+                                      "Release a terminated publication with 2xx\n"));
+              REMOVE_ELEMENT (eXosip.j_pub, jpub);
+              _eXosip_pub_free (jpub);
+            }
+        }
+
+      jpub = jpubnext;
+    }
+
+}
